@@ -259,38 +259,6 @@ class SQLiteModel(data_algebra.db_model.DBModel):
         # https://docs.python.org/3/library/sqlite3.html
         conn.create_aggregate("median", 1, MedianAgg)
 
-    # noinspection PyMethodMayBeStatic,SqlNoDataSourceInspection
-    def insert_table(
-        self, conn, d, table_name, *, qualifiers=None, allow_overwrite=False
-    ):
-        """
-
-        :param conn: a database connection
-        :param d: a Pandas table
-        :param table_name: name to give write to
-        :param qualifiers: schema and such
-        :param allow_overwrite logical, if True drop previous table
-        """
-
-        if qualifiers is not None:
-            raise ValueError("non-empty qualifiers not yet supported on insert")
-        cur = conn.cursor()
-        # check for table
-        table_exists = True
-        # noinspection PyBroadException
-        try:
-            self.read_query(conn, "SELECT * FROM " + table_name + " LIMIT 1")
-        except Exception:
-            table_exists = False
-        if table_exists:
-            if not allow_overwrite:
-                raise ValueError("table " + table_name + " already exists")
-            else:
-                cur.execute("DROP TABLE " + table_name)
-        # Note: the Pandas to_sql() method appears to have SQLite hard-wired into it
-        # it refers to sqlite_master
-        d.to_sql(name=table_name, con=conn, index=False)
-
     def _emit_right_join_as_left_join(
         self, join_node, *, using=None, temp_id_source, sql_format_options=None
     ):
@@ -388,7 +356,7 @@ class SQLiteModel(data_algebra.db_model.DBModel):
         )
 
 
-def example_handle():
+def example_handle() -> data_algebra.db_model.DBHandle:
     """
     Return an example db handle for testing. Returns None if helper packages not present.
 
