@@ -3,7 +3,7 @@ Represent data processing expressions.
 """
 
 from abc import ABC
-from typing import Optional, Union
+from typing import Any, List, Optional, Union
 
 import numpy
 
@@ -24,6 +24,9 @@ class PythonText:
     str() method returns only the text for interoperability.
     """
 
+    s: str
+    is_in_parens: bool
+
     def __init__(self, s: str, *, is_in_parens: bool = False):
         assert isinstance(s, str)
         assert isinstance(is_in_parens, bool)
@@ -61,6 +64,7 @@ fn_names_that_imply_windowed_situation = {
     "median",
     "min",
     "ngroup",
+    "_ngroup",
     "nlargest",
     "nsmallest",
     "nth",
@@ -96,6 +100,7 @@ fn_names_that_imply_ordered_windowed_situation = {
 # noinspection SpellCheckingInspection
 fn_names_not_allowed_in_project = {
     "ngroup",
+    "_ngroup",
 }
 
 
@@ -359,19 +364,6 @@ class Term(PreTerm, ABC):
     def __pos__(self):
         return self  # Treat as a no-op
 
-    # # TODO: need to work out how to send to Pandas and SQL
-    # def __lshift__(self, other):
-    #     return self.__op_expr__("<<", other)
-    #
-    # def __rlshift__(self, other):
-    #     return self.__rop_expr__("<<", other)
-    #
-    # def __rshift__(self, other):
-    #     return self.__op_expr__(">>", other)
-    #
-    # def __rrshift__(self, other):
-    #     return self.__rop_expr__(">>", other)
-
     def __and__(self, other):
         return self.__op_expr__("&", other)
 
@@ -397,7 +389,7 @@ class Term(PreTerm, ABC):
     # math functions
     # https://docs.scipy.org/doc/numpy-1.13.0/reference/routines.math.html
 
-    # TODO: double check https://docs.python.org/3/library/operator.html
+    # Python operators: https://docs.python.org/3/library/operator.html
 
     def sign(self):
         """
@@ -692,12 +684,6 @@ class Term(PreTerm, ABC):
         """
         return self.__uop_expr__("min")
 
-    def ngroup(self):
-        """
-        Return number of groups (vectorized).
-        """
-        return self.__uop_expr__("ngroup")
-
     def nunique(self):
         """
         Return number of unique items (vectorized).
@@ -1180,7 +1166,7 @@ def enc_value(value):
 class ColumnReference(Term):
     """class to represent referring to a column"""
 
-    view: Optional["data_algebra.data_ops.ViewRepresentation"]
+    view: Optional[Any]
     column_name: str
 
     def __init__(self, view, column_name):
@@ -1252,14 +1238,11 @@ def _can_find_method_by_name(op):
     # from populate_specials
     if op in {
         "_count",
-        "count",
-        "row_number",
         "_row_number",
         "_size",
-        "size",
-        "connected_components",
+        "_connected_components",
         "_ngroup",
-        "ngroup",
+        "_uniform",
     }:
         return True
     # check user fns
@@ -1392,6 +1375,12 @@ class Expression(Term):
             return method_to_call(*args)
         except KeyError:
             pass
+        # special zero argument functions
+        if len(args) == 0:
+            if self.op == '_uniform':
+                return numpy.random.uniform(size=data_frame.shape[0])
+            else:
+                KeyError(f"zero-argument function {self.op} not found")
         # now see if argument (usually Pandas) can do this
         # doubt we hit in this, as most exposed methods are window methods
         try:
@@ -1434,27 +1423,27 @@ class Expression(Term):
                     "(" + str(sub_0) + ")." + self.op + "()", is_in_parens=False
                 )
         if self.inline:
-            subs = [str(ai.to_python(want_inline_parens=True)) for ai in self.args]
-            result = (" " + self.op + " ").join(subs)
+            subs_strs = [str(ai.to_python(want_inline_parens=True)) for ai in self.args]
+            result = (" " + self.op + " ").join(subs_strs)
             if want_inline_parens:
                 return PythonText("(" + result + ")", is_in_parens=True)
             return PythonText(result, is_in_parens=False)
-        subs = [ai.to_python(want_inline_parens=False) for ai in self.args]
+        subs: List[PythonText] = [ai.to_python(want_inline_parens=False) for ai in self.args]
         subs_0 = subs[0]
-        subs = [str(si) for si in subs]
+        subs_strs = [str(si) for si in subs]
         if self.method:
             if subs_0.is_in_parens or isinstance(self.args[0], ColumnReference):
                 return PythonText(
-                    subs[0] + "." + self.op + "(" + ", ".join(subs[1:]) + ")",
+                    subs_strs[0] + "." + self.op + "(" + ", ".join(subs_strs[1:]) + ")",
                     is_in_parens=False,
                 )
             else:
                 return PythonText(
-                    "(" + subs[0] + ")." + self.op + "(" + ", ".join(subs[1:]) + ")",
+                    "(" + subs_strs[0] + ")." + self.op + "(" + ", ".join(subs_strs[1:]) + ")",
                     is_in_parens=False,
                 )
         # treat as fn call
-        return PythonText(self.op + "(" + ", ".join(subs) + ")", is_in_parens=False)
+        return PythonText(self.op + "(" + ", ".join(subs_strs) + ")", is_in_parens=False)
 
 
 # define with def so function has usable __name__
