@@ -30,9 +30,10 @@ class OpC(data_algebra.data_ops_types.OperatorPlatform):
         data_algebra.data_ops_types.OperatorPlatform.__init__(
             self, node_name="container"
         )
+        self.ops = None
         self.set(other)
 
-    def set(self, other):
+    def set(self, other: data_algebra.data_ops.ViewRepresentation):
         assert isinstance(other, data_algebra.data_ops.ViewRepresentation)
         assert not isinstance(other, OpC)  # don't allow deep nesting
         self.ops = other
@@ -67,8 +68,13 @@ class OpC(data_algebra.data_ops_types.OperatorPlatform):
         return res
 
     # noinspection PyPep8Naming
-    def transform(self, X, *, data_model=None, narrow=True):
-        return self.ops.transform(X=X, data_model=data_model, narrow=narrow)
+    def transform(self, X, *, data_model=None, narrow: bool = True, check_incoming_data_constraints: bool = False):
+        assert isinstance(self.ops, data_algebra.data_ops.ViewRepresentation)
+        return self.ops.transform(
+            X=X,
+            data_model=data_model,
+            narrow=narrow,
+            check_incoming_data_constraints=check_incoming_data_constraints)
 
     # noinspection PyPep8Naming
     def act_on(self, X, *, data_model=None):
@@ -105,18 +111,18 @@ class OpC(data_algebra.data_ops_types.OperatorPlatform):
 
     # query generation
 
-    def to_near_sql_implementation(self, db_model, *, using, temp_id_source):
-        return self.ops.to_near_sql_implementation(
+    def to_near_sql_implementation_(self, db_model, *, using, temp_id_source):
+        return self.ops.to_near_sql_implementation_(
             db_model=db_model, using=using, temp_id_source=temp_id_source
         )
 
     # define builders for all non-initial ops types on base class
 
-    def extend_parsed(
+    def extend_parsed_(
         self, parsed_ops, *, partition_by=None, order_by=None, reverse=None
     ):
         self.set(
-            self.ops.extend_parsed(
+            self.ops.extend_parsed_(
                 parsed_ops=parsed_ops,
                 partition_by=partition_by,
                 order_by=order_by,
@@ -133,8 +139,8 @@ class OpC(data_algebra.data_ops_types.OperatorPlatform):
         )
         return self
 
-    def project_parsed(self, parsed_ops=None, *, group_by=None):
-        self.set(self.ops.project_parsed(parsed_ops=parsed_ops, group_by=group_by))
+    def project_parsed_(self, parsed_ops=None, *, group_by=None):
+        self.set(self.ops.project_parsed_(parsed_ops=parsed_ops, group_by=group_by))
         return self
 
     def project(self, ops=None, *, group_by=None):
@@ -158,8 +164,8 @@ class OpC(data_algebra.data_ops_types.OperatorPlatform):
         )
         return self
 
-    def select_rows_parsed(self, parsed_expr):
-        self.set(self.ops.select_rows_parsed(parsed_expr=parsed_expr))
+    def select_rows_parsed_(self, parsed_expr):
+        self.set(self.ops.select_rows_parsed_(parsed_expr=parsed_expr))
         return self
 
     def select_rows(self, expr):
