@@ -148,9 +148,19 @@ def equivalent_frames(
                 if ca_can_be_numeric and cb_can_be_numeric:
                     if len(ca_n) != len(cb_n):
                         return False
-                    dif = numpy.abs(ca_n - cb_n) / numpy.maximum(numpy.maximum(numpy.abs(ca_n), numpy.abs(cb_n)), 1.0)
-                    if numpy.max(dif) > float_tol:
+                    ca_inf = numpy.isinf(ca_n)
+                    cb_inf = numpy.isinf(cb_n)
+                    if numpy.any(ca_inf != cb_inf):
                         return False
+                    if numpy.any(ca_inf):
+                        if numpy.any(numpy.sign(ca_n[ca_inf]) != numpy.sign(cb_n[cb_inf])):
+                            return False
+                    if numpy.any(numpy.logical_not(ca_inf)):
+                        ca_f = ca_n[numpy.logical_not(ca_inf)]
+                        cb_f = cb_n[numpy.logical_not(cb_inf)]
+                        dif = numpy.abs(ca_f - cb_f) / numpy.maximum(numpy.maximum(numpy.abs(ca_f), numpy.abs(cb_f)), 1.0)
+                        if numpy.max(dif) > float_tol:
+                            return False
                 else:
                     if not numpy.all(ca == cb):
                         return False
@@ -273,7 +283,7 @@ def _run_handle_experiments(
 def check_transform_on_handles(
     *,
     ops,
-    data,
+    data: dict,
     expect,
     db_handles,
     float_tol: float = 1e-8,
@@ -282,6 +292,7 @@ def check_transform_on_handles(
     check_row_order: bool = False,
     check_parse: bool = True,
     local_data_model=None,
+    valid_for_empty: bool = True,
     empty_produces_empty: bool = True,
 ) -> None:
     """
@@ -298,16 +309,13 @@ def check_transform_on_handles(
     :param check_row_order: passed to equivalent_frames()
     :param check_parse: if True check expression parses/formats to self
     :param local_data_model: optional alternate evaluation model
-    :param empty_produces_empty: logical, if true assume emtpy inputs should produce empty output
+    :param valid_for_empty: logical, if True test on empty inputs
+    :param empty_produces_empty: logical, if True assume emtpy inputs should produce empty output
     :return: None, assert if there is an issue
     """
 
-    # convert single table to dictionary
-    if not isinstance(data, dict):
-        cols_used = ops.columns_used()
-        table_name = [k for k in cols_used.keys()][0]
-        data = {table_name: data}
     assert isinstance(data, dict)
+    orig_data = {k: v.copy() for k, v in data.items()}
     n_tables = len(data)
     assert n_tables > 0
     if local_data_model is None:
@@ -337,6 +345,10 @@ def check_transform_on_handles(
         check_row_order=check_row_order,
     ):
         raise ValueError("Pandas eval result did not match expect")
+    # show inputs didn't change
+    for k, v in orig_data.items():
+        v2 = data[k]
+        assert equivalent_frames(v, v2)
     if check_parse:
         if not formats_to_self(ops):
             raise ValueError("ops did not round-trip format")
@@ -366,34 +378,39 @@ def check_transform_on_handles(
             check_row_order=check_row_order,
         ):
             raise ValueError("Pandas transform result did not match expect")
-    # try on empty inputs
-    empty_map = {
-        k: v.iloc[range(0), :].reset_index(drop=True, inplace=False)
-        for k, v in data.items()
-    }
-    empty_res = ops.eval(empty_map)
-    assert local_data_model.is_appropriate_data_instance(empty_res)
-    assert set(empty_res.columns) == set(res.columns)
-    if empty_produces_empty:
-        assert empty_res.shape[0] == 0
-    else:
-        assert empty_res.shape[0] > 0
-    # try on combinations of empty and original
-    if n_tables == 2:
-        keys = [k for k in data.keys()]
-        partial_maps = [
-            {
-                keys[0]: data[keys[0]].reset_index(drop=True, inplace=False),
-                keys[1]: data[keys[1]],
-            },
-            {
-                keys[0]: data[keys[0]],
-                keys[1]: data[keys[1]].reset_index(drop=True, inplace=False),
-            },
-        ]
-        for pm in partial_maps:
-            empty_res_i = ops.eval(pm)
-            assert set(empty_res_i.columns) == set(res.columns)
+        # show inputs didn't change
+        for k, v in orig_data.items():
+            v2 = data[k]
+            assert equivalent_frames(v, v2)
+    if valid_for_empty:
+        # try on empty inputs
+        empty_map = {
+            k: v.iloc[range(0), :].reset_index(drop=True, inplace=False)
+            for k, v in data.items()
+        }
+        empty_res = ops.eval(empty_map)
+        assert local_data_model.is_appropriate_data_instance(empty_res)
+        assert set(empty_res.columns) == set(res.columns)
+        if empty_produces_empty:
+            assert empty_res.shape[0] == 0
+        else:
+            assert empty_res.shape[0] > 0
+        # try on combinations of empty and original
+        if n_tables == 2:
+            keys = [k for k in data.keys()]
+            partial_maps = [
+                {
+                    keys[0]: data[keys[0]].reset_index(drop=True, inplace=False),
+                    keys[1]: data[keys[1]],
+                },
+                {
+                    keys[0]: data[keys[0]],
+                    keys[1]: data[keys[1]].reset_index(drop=True, inplace=False),
+                },
+            ]
+            for pm in partial_maps:
+                empty_res_i = ops.eval(pm)
+                assert set(empty_res_i.columns) == set(res.columns)
     # try any db paths
     global global_test_result_cache
     global run_direct_ops_path_tests
@@ -475,6 +492,7 @@ def check_transform(
     check_row_order: bool = False,
     check_parse: bool = True,
     models_to_skip: Optional[Iterable[str]] = None,
+    valid_for_empty: bool = True,
     empty_produces_empty: bool = True,
 ) -> None:
     """
@@ -490,7 +508,8 @@ def check_transform(
     :param check_row_order: passed to equivalent_frames()
     :param check_parse: if True check expression parses/formats to self
     :param models_to_skip: None or set of model names to skip testing
-    :param empty_produces_empty: logical, if true assume emtpy inputs should produce empty output
+    :param valid_for_empty: logical, if True perform tests on empty inputs
+    :param empty_produces_empty: logical, if True assume emtpy inputs should produce empty output
     :return: nothing
     """
 
@@ -528,6 +547,7 @@ def check_transform(
             check_row_order=check_row_order,
             check_parse=check_parse,
             db_handles=db_handles,
+            valid_for_empty=valid_for_empty,
             empty_produces_empty=empty_produces_empty,
         )
     except AssertionError:
